
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from qgis.core import QgsContextHelp

from ui_imagemapplugingui import Ui_ImageMapPluginGui

import imagemapplugin_rc


class ImageMapPluginGui(QDialog, Ui_ImageMapPluginGui):

  PATH_STRING = "Path and file name (without extension)"
  MSG_BOX_TITLE = "QGis Image Map Plugin"

  def __init__(self, parent, fl):
    QDialog.__init__(self, parent, fl)
    self.setupUi(self)
    self.emit(SIGNAL("getAttribute(QString)"), self.cmbAttributesHref.currentText() )

  def on_buttonBox_accepted(self):
    #print "OK button in buttonbox"
    # check to see if at least one of the attr checkboxes are checked
    if (not self.chkBoxHref.isChecked()) and (not self.chkBoxOnClick.isChecked()) and (not self.chkBoxOnMouseOver.isChecked()):
      QMessageBox.warning(self, self.MSG_BOX_TITLE, ("Not one attr checkbox checked?\n" "Please choose at least one attribute to use in AREA tags."), QMessageBox.Ok)
      return
    self.emit(SIGNAL("getFilesPath(QString)"), self.txtFileName.text() )
    self.emit(SIGNAL("onHrefAttributeSet(QString)"), self.cmbAttributesHref.currentText() )
    self.emit(SIGNAL("onClickAttributeSet(QString)"), self.cmbAttributesOnClick.currentText() )
    self.emit(SIGNAL("onMouseOverAttributeSet(QString)"), self.cmbAttributesOnMouseOver.currentText() )
    # and GO
    self.emit(SIGNAL("go(QString)"), "ok" )
    #self.done(1)   

  def on_buttonBox_rejected(self):
    #print "CANCEL button in buttonbox"
    self.done(0)

  def on_buttonBox_helpRequested(self):
    # print "HELP button in buttonbox"
    # TODO make help resources ?
    QgsContextHelp.run(32338213)

  def on_chkBoxSelectedOnly_stateChanged(self):
    self.emit(SIGNAL("getCbkBoxSelectedOnly(bool)"), self.chkBoxSelectedOnly.isChecked() )

  def on_chkBoxHref_stateChanged(self):
    self.cmbAttributesHref.setEnabled(self.chkBoxHref.isChecked())

  def on_chkBoxOnClick_stateChanged(self):
    self.cmbAttributesOnClick.setEnabled(self.chkBoxOnClick.isChecked())

  def on_chkBoxOnMouseOver_stateChanged(self):
    self.cmbAttributesOnMouseOver.setEnabled(self.chkBoxOnMouseOver.isChecked())

  # if the text in this field is stil beginning with: 'full path and name'
  def on_txtFileName_cursorPositionChanged(self, old, new):
    if self.txtFileName.text().startsWith(self.PATH_STRING):  # text() returns QString => startsWith instead startswith
        self.txtFileName.setText('')

  # see http://www.riverbankcomputing.com/Docs/PyQt4/pyqt4ref.html#connecting-signals-and-slots
  # without this magic, the on_btnOk_clicked will be called two times: one clicked() and one clicked(bool checked)
  @pyqtSignature("on_btnBrowse_clicked()")
  def on_btnBrowse_clicked(self):
    fileName = QFileDialog.getSaveFileName(self, self.PATH_STRING, "/", "")
    # TODO do some checks to be sure there is no extension
    self.txtFileName.setText(fileName)

  def setFilesPath(self, path):
    self.txtFileName.setText(path)

  def setAttributeFields(self, layerAttr):
    # populate comboboxes with attribute field names of active layer
    self.cmbAttributesHref.addItems(layerAttr)
    self.cmbAttributesOnClick.addItems(layerAttr)
    self.cmbAttributesOnMouseOver.addItems(layerAttr)

  def setProgressBarMax(self, maxInt):
    # minimum default to zero
    self.progressBar.setMinimum(0)
    self.progressBar.setMaximum(maxInt)

  def setProgressBarValue(self, valInt):
    self.progressBar.setValue(valInt)

  def isHrefChecked(self):
    return self.chkBoxHref.isChecked()

  def isOnClickChecked(self):
    return self.chkBoxOnClick.isChecked()

  def isOnMouseOverChecked(self):
    return self.chkBoxOnMouseOver.isChecked()
