
import os

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from qgis.core import *

from imagemapplugingui import ImageMapPluginGui


# initialize Qt resources from file
import imagemapplugin_rc

class ImageMapPlugin:

  MSG_BOX_TITLE = "QGis Image Map Plugin"

  def __init__(self, iface):
    # save reference to the QGIS interface
    self.iface = iface
    self.filesPath = "/home/"

  def initGui(self):
    # create action that will start plugin configuration
    self.action = QAction(QIcon(":/imagemapicon.xpm"), "Image Map", self.iface.getMainWindow())
    self.action.setWhatsThis("Configuration for Image Map plugin")
    QObject.connect(self.action, SIGNAL("activated()"), self.run)
    # add toolbar button and menu item
    self.iface.addToolBarIcon(self.action)
    self.iface.addPluginMenu("&Html Image Map Plugin", self.action)
    # connect to signal renderComplete which is emitted when canvas rendering is done
    QObject.connect(self.iface.getMapCanvas(), SIGNAL("renderComplete(QPainter *)"), self.renderTest)

  def unload(self):
    # remove the plugin menu item and icon
    self.iface.removePluginMenu("&Html Image Map Plugin",self.action)
    self.iface.removeToolBarIcon(self.action)
    # disconnect form signal of the canvas
    QObject.disconnect(self.iface.getMapCanvas(), SIGNAL("renderComplete(QPainter *)"), self.renderTest)

  def run(self):
    # check if current active layer is a polygon layer:
    layer =  self.iface.activeLayer()
    if layer == None:
        #print "Uhm, no active layer, make one polygon layer active"
        QMessageBox.warning(self.iface.getMainWindow(), self.MSG_BOX_TITLE, ("No active layer found\n" "Please make one (multi)polygon layer active\n" "by choosing a layer in the legend"), QMessageBox.Ok, QMessageBox.Ok)
        return
    # don't know if this is possible / needed
    if not layer.isValid():
        #print "uhm, layer not valid, no active layer?"
        QMessageBox.warning(self.iface.getMainWindow(), self.MSG_BOX_TITLE, ("No VALID layer found\n" "Please make one (multi)polygon layer active\n" "by choosing a layer in the legend"), QMessageBox.Ok, QMessageBox.Ok)
        return
    self.provider = layer.getDataProvider()
    self.currentExtent = self.iface.getMapCanvas().extent()
    if self.provider.geometryType() <> 3: # 3: polygons
        #print "Wrong geometrytype, only polygons (3) allowed, but is: %s" % provider.geometryType()
        QMessageBox.warning(self.iface.getMainWindow(), self.MSG_BOX_TITLE, ("Wrong geometrytype, only (multi)polygons can be used.\n" "Please make one (multi)polygon layer active\n" "by choosing a layer in the legend"), QMessageBox.Ok, QMessageBox.Ok)
        return

    # we need the fields of the active layer to show in the attribute combobox in the gui:
    fields = self.provider.fields()
    attrFields = []
    for (i, field) in fields.iteritems():
      attrFields.append(field.name().trimmed())
    # construct gui (using these fields)
    flags = Qt.WindowTitleHint | Qt.WindowSystemMenuHint | Qt.WindowMaximizeButtonHint  # QgisGui.ModalDialogFlags
    # construct gui: if available reuse this one
    if hasattr(self, 'imageMapPlugin') == False:
        self.imageMapPluginGui = ImageMapPluginGui(self.iface.getMainWindow(), flags)
    self.imageMapPluginGui.setAttributeFields(attrFields)
    self.layerAttr = attrFields
    self.selectedFeaturesOnly = False # default all features in current Extent
    # poging om een SIGNAL te vangen
    QObject.connect(self.imageMapPluginGui, SIGNAL("getFilesPath(QString)"), self.setFilesPath)
    QObject.connect(self.imageMapPluginGui, SIGNAL("onHrefAttributeSet(QString)"), self.onHrefAttributeFieldSet)
    QObject.connect(self.imageMapPluginGui, SIGNAL("onClickAttributeSet(QString)"), self.onClickAttributeFieldSet)
    QObject.connect(self.imageMapPluginGui, SIGNAL("onMouseOverAttributeSet(QString)"), self.onMouseOverAttributeFieldSet)
    QObject.connect(self.imageMapPluginGui, SIGNAL("getCbkBoxSelectedOnly(bool)"), self.setSelectedOnly)
    QObject.connect(self.imageMapPluginGui, SIGNAL("go(QString)"), self.go)
    # remember old path's in this session:
    self.imageMapPluginGui.setFilesPath(self.filesPath)
    self.imageMapPluginGui.show()


  def writeHtml(self):
    # create a holder for retrieving features from the provider
    feature = QgsFeature();
    temp = str(self.filesPath+".png")
    imgfilename = os.path.basename(temp)
    html = ['<html>']
    # some rudimentary javascript to show off the mouse click and mouse over
    html.append('<head><script>\n')
    html.append('function mapOnMouseOver(str){document.getElementById("mousemovemessage").innerHTML=str; }\n')
    html.append('function mapOnClick(str){alert(str);}\n')
    html.append('</script></head><body>')
    html.append('<div id="mousemovemessage"></div><br/>')
    html.append('<img src="' + imgfilename + '" border="0" ismap="1" usemap="#mapmap" >\n')
    html.append('<map name="mapmap">\n')
    # now iterate through each feature
    # NOTE: this already only returns the features in current view
    currentExtent = self.currentExtent
    # keep the current extent as a Geometry to be able to do some 'contains' tests later
    self.extentAsPoly = QgsGeometry.fromRect(currentExtent)
    # select features within current extent,
    #   with  ALL attributes, WITHIN currentExtent, WITH geom, AND using Intersect instead of bbox
    self.provider.select(self.provider.allAttributesList(), currentExtent, True, True)
    # get a list of all selected features ids
    selectedFeaturesIds = self.iface.activeLayer().selectedFeaturesIds()
    # set max progress bar to number of features (not very accurate with a lot of huge multipolygons)
    #self.imageMapPluginGui.setProgressBarMax(self.iface.activeLayer().featureCount())
    # or run over all features in current selection, just to determine the number of... (should be simpler ...)
    count = 0
    while self.provider.getNextFeature(feature):
        count = count + 1
    self.imageMapPluginGui.setProgressBarMax(count)
    progressValue = 0

    # it seems that a postgres provider is on the end of file now
    # we do the select again to set the pointer/cursor to 0 again ?
    self.provider.select(self.provider.allAttributesList(), currentExtent, True, True)
    while self.provider.getNextFeature(feature):
      progressValue = progressValue+1
      self.imageMapPluginGui.setProgressBarValue(progressValue)
      # if checkbox 'selectedFeaturesOnly' is checked: check if this feature is selected
      if self.selectedFeaturesOnly and feature.featureId() not in selectedFeaturesIds:
        # print "skipping %s " % feature.featureId()
        None
      else:
        geom = feature.geometry()
        if geom.wkbType() == QGis.WKBPolygon: # 3 = WKBTYPE.WKBPolygon:
            polygon = geom.asPolygon()  # returns a list
            for ring in polygon:
                h = self.ring2area(feature, ring, currentExtent)
                html.append(h)
        if geom.wkbType() == QGis.WKBMultiPolygon: # 6 = WKBTYPE.WKBMultiPolygon:
            multipolygon = geom.asMultiPolygon() # returns a list
            for polygon in multipolygon:
                for ring in polygon:
                    h = self.ring2area(feature, ring, currentExtent)
                    html.append(h)
    html.append('</map></body></html>')
    return html

  def renderTest(self, painter):
    # Get canvas dimensions
    self.canvaswidth = painter.device().width()
    self.canvasheight = painter.device().height()

  def setFilesPath(self, filesPathQString):
    self.filesPath = filesPathQString

  def onHrefAttributeFieldSet(self, attributeFieldQstring):
    self.hrefAttributeField = attributeFieldQstring
    self.hrefAttributeIndex = self.provider.indexFromFieldName(attributeFieldQstring)

  def onClickAttributeFieldSet(self, attributeFieldQstring):
    self.onClickAttributeField = attributeFieldQstring
    self.onClickAttributeIndex = self.provider.indexFromFieldName(attributeFieldQstring)

  def onMouseOverAttributeFieldSet(self, attributeFieldQstring):
    self.onMouseOverAttributeField = attributeFieldQstring
    self.onMouseOverAttributeIndex = self.provider.indexFromFieldName(attributeFieldQstring)

  def setSelectedOnly(self, selectedOnlyBool):
    print "selectedFeaturesOnly: %s" % selectedOnlyBool
    self.selectedFeaturesOnly = selectedOnlyBool

  def go(self, foo):
    htmlfilename = self.filesPath + ".html"
    imgfilename = self.filesPath + ".png"
    # check if path is writable: ?? TODO
    #if not os.access(htmlfilename, os._OK):
    #  QMessageBox.warning(self.iface.getMainWindow(), self.MSG_BOX_TITLE, ("Unable to write file with this name.\n" "Please choose a valid filename and a writable directory."))
    #  return
    # check if file(s) excist:
    if os.path.isfile(htmlfilename) or os.path.isfile(imgfilename):
        if QMessageBox.question(self.iface.getMainWindow(), self.MSG_BOX_TITLE, ("There is already a filename with this name.\n" "Continue?"), QMessageBox.Cancel, QMessageBox.Ok) <> QMessageBox.Ok:
            return
    # else: everthing ok: start writing img and html
    file = open(htmlfilename, "w")
    html = self.writeHtml()
    file.writelines(html)
    file.close()
    self.iface.getMapCanvas().saveAsImage(imgfilename)
    self.imageMapPluginGui.hide()

  # NOT WORKING ????
  # pixpoint = m2p.transform(point.x(), point.y())
  # print m2p.transform(point.x(), point.y())
  # so for now: a custom 'world2pixel' method
  def w2p(self, x, y, mupp, minx, maxy):
    pixX = (x - minx)/mupp
    pixY = (y - maxy)/mupp
    return [int(pixX), int(-pixY)]

  # for given ring in feature, IF al least on point on ring is in currentExtent
  # generate a string like:
  # <area shape=polygon href='xxx' onClick="mapOnClick('yyy')" onMouseOver="mapOnMouseOver('zzz')  coords=519,-52,519,..,-52,519,-52>
  def ring2area(self, feature, ring, currentExtent):
    htm = '<area shape="polygon" '
    if self.imageMapPluginGui.isHrefChecked():
        htm = htm + 'href="' + feature.attributeMap()[self.hrefAttributeIndex].toString() + '" '
    if self.imageMapPluginGui.isOnClickChecked():
        # remove ' and " because the will collapse as javascript parameter
        param = feature.attributeMap()[self.onClickAttributeIndex].toString()
        htm = htm + 'onClick="mapOnClick(\'' + self.jsEscapeString(param) + '\')" '
    if self.imageMapPluginGui.isOnMouseOverChecked():
        # remove ' and " because the will collapse as javascript parameter
        param = feature.attributeMap()[self.onMouseOverAttributeIndex].toString()
        htm = htm + 'onMouseOver="mapOnMouseOver(\'' + self.jsEscapeString(param) + '\')" '
    htm = htm + ' coords="'
    lastPixel=[0,0]
    insideExtent = False
    coordCount = 0
    for point in ring:
        # NOT WORKING ????
        #pixpoint = m2p.transform(point.x(), point.y())
        #print m2p.transform(point.x(), point.y())
        # this should be easier to do? Check if all points if this ring are withing current extent, only if true add to html
        if self.extentAsPoly.contains(point):
            insideExtent = True
        pixpoint =  self.w2p(point.x(), point.y(), self.iface.getMapCanvas().mupp(),currentExtent.xMin(),currentExtent.yMax())
        if lastPixel<>pixpoint:
            coordCount = coordCount +1
            htm += (str(pixpoint[0]) + ',' + str(pixpoint[1]) + ',')
            lastPixel = pixpoint
    htm.remove(len(htm)-1,len(htm))
    # check if there are more then 2 coords: very small polygons on current map can have coordinates
    # which if rounded to pixels all come to the same pixel, resulting in just ONE x,y coordinate
    # we skip these
    if coordCount < 2:
        #print "Ring contains just one pixel coordinate pair: skipping"
        return ''
    # if at least ONE pixel of this ring is in current view extent, return the area-string, otherwise return an empty string
    if not insideExtent:
        #print "RING FULLY OUTSIDE EXTENT: %s " % ring
        return ''
    else:
        htm += '">\n'
        return htm


  # escape ' and " so string can be safely used as string javascript argument
  def jsEscapeString(self, str):
    return str.replace("'", "\\'").replace('"', '\"')
